<?php

    class HTTP_Get_Post_Evaluation_Action extends Reactor_Action
    {
        protected $tpl = 'http_get_post_evaluation.php';
        protected $name = 'Submit an HTTP request';
		
		public static $registration = array(
			ACTION_NAME => 'Submits HTTP GET/POST request',
			ACTION_CATEGORY => 'Network',
			ACTION_DESCRIPTION => 'Submits an HTTP GET/POST request and evaluates the return code.',
			ACTION_AUTHOR => 'Jeremy Yunis, Nagios Enterprises, LLC',
			ACTION_VERSION => '1.0',
			ACTION_DATE => '01/22/2013', //Last version update
			ACTION_AUTHLEVEL => 0,  // 0 - everyone | 1 - Admin only
			ACTION_TYPE => 'core', //core | user. core modules can't be deleted
			ACTION_META => array('http_result_code'),	//Misc information	
		); 
		
        protected $fields = array(
            'url' => array(
                'label' => 'url',
                'validate' => 'trim|required|xss_clean|callback_check_url',
            ),
            'request_type' => array(
                'label' => 'request type',
                'validate' => 'trim|required|xss_clean|alpha|min_length[3]|max_length[4]'
            ),
            'extra_params' => array(
                'label' => 'get/post params',
                'validate' => 'trim|xss_clean|prep_for_form',
            ),
        );

        /** 
         * Form validation function to check a url
         **/
        public function check_url($str)
        {
            $url_expression = "/^(https?|ftp)\:\/\/([a-z0-9+!*(),;?&=\$_.-]+(\:[a-z0-9+!*(),;?&=\$_.-]+)?@)?[a-z0-9+\$_-]+(\.[a-z0-9+\$_-]+)*(\:[0-9]{2,5})?(\/([a-z0-9+\$_-]\.?)+)*\/?(\?[a-z+&\$_.-][a-z0-9;:@\/&%=+\$_.-]*)?(#[a-z_.-][a-z0-9+\$_.-]*)?\$/i";
            if(!preg_match($url_expression, $str)) {
                $this->ci->form_validation->set_message('check_url', 'The url specified is invalid.');
                return FALSE;
            } else {
                return TRUE;
            }
        }

        public function get_label()
        {
            if($params = $this->get_params()) {
                if(array_key_exists('request_type', $params)) {
                    return sprintf('Submit an HTTP %s request to \'%s\'', $params['request_type'], $params['url']);
                }
            }
            return parent::get_label();
        }
       
        public function run($context)
        {
            $this->set_context($context);
            $params = $this->get_params();
            if(!in_array('curl', get_loaded_extensions())) {
                $this->log('error', 'cURL is not installed and is required for this function.');
                $this->set_result(FALSE);
                return;
            }
            $url = $params['url'];
            $post_query = FALSE;
            if(array_key_exists('extra_params', $params)) {
                $extra_params = $params['extra_params'];
                if(strtolower($params['request_type']) == 'post') {
                    $post_query = $extra_params;
                } else {
                    if (strpos($url, '?' === false))
						$url .= '?'.$extra_params;
					else
						$url .= '&'.$extra_params;
                }
            }
            $this->log('info', sprintf('Performing %s request to %s', $params['request_type'], $url));
            list($http_result, $http_info) = do_curl_request($url, $post_query);
            $this->log('info', 'HTTP status was '.$http_info['http_code']);
			$this->set_context_var('http_result_code', $http_info['http_code']);	
            $s = $http_info['http_code'];
            $this->set_result(($s >= 200 && $s < 300) ? 0 : 1);
            return $context;
        }
    }
